/* Counter.c
* 	Winford Engineering
* 	www.winfordeng.com
* 	Revised: October 1999
* 	By: JAB
* 	
*	This a sample program for Winford Engineering's WECRD155B I/O Card.
*	This program uses Winford Engineering's I/O Card API
*	Hardware setup:
*		This program assumes that you have eight LEDs connected
*		to port 1 (port A) of your I/O card.  It also assumes that you have
*		a bounceless pushbutton (or a low frequency clock source) 
*		connected between Bit 0 of port 2 (port B) and ground.
*	Operation:
*		This program keeps a count of how many times the pushbutton
*		has been pressed.  The count begins at 0 and is incremented
*		every time the button is pressed.  To monitor the state of the
*		pushbutton, the program configures Port 2 to be in the input state.
*		The current value is output onto Port 1; if you have LEDs connected
*		you will be able to see the value of the counter in binary form.
*		Notice that since Port 1 is configured for output operation, you 
*		should take care that you do not have any circuitry connected
*		to port 1 that could interfere with the port's outputs (See the
*		section in your manual about Data Contention).
*	Compiling Instructions:
*		This program is written for Winford Engineering's I/O Card api.
*		To successfully compile this code, you must have the header file
*		for the API (wecrd.h) and the library file (wecrd.a in linux,
*		wecrd.lib and wecrd.dll in windows) See the API documentation
*		for more details (Support section of www.winfordeng.com).
*				
*/

#include <stdio.h>
#include "wecrd.h"

int main()
{
	long result; //holds the error or success code from API calls
	long handle; //holds the identifier returned by we_OpenCard.
	int count=0; //counter of how many times the button has been pressed
	long value;  //value read from Port 2
	char choice; 
	
	//First, we will output a notice and ask for user confirmation
	//this would not normally be done in an actual application.
	printf("Note: This program configures Port 1 as an output port.  Please be sure that you do not have interfering hardware connected to port 1 of your card.\nContinue (Y/N)?");
	choice = getchar();
	if( (choice != 'y') && (choice != 'Y') )
		return(0);
	
	//Open the card.  This should be the first API call in every program.
	//If your card is not configured for base address 384, edit the line below
	handle = we_OpenCard(WECRD_155B, 384);
	if(handle < 0)//error occurred
	{
		printf("An error occurred while opening the card. Check that your card is configured for the base address used in the source code.\n");
		return(-1);
	}
	
	//We now need to configure Port 1 (A) to be in output mode
	result = we_ConfigureCard(handle, 1, WECRD_OUTPUT);
	if(result < 0)//Note that in a normal application, it would not be necessary to 
	{			  // check for errors on all of the functions, but it is done here
				  // to show that it is possible.
		printf("An error occurred while configuring the card.\n");
		we_CloseCard(handle);//release card handle before exiting
		return(-1);
	}

	//We now need to configure Port 2 (B) to be in input mode
	result = we_ConfigureCard(handle, 2, WECRD_INPUT);
	if(result < 0)
	{
		printf("An error occurred while configuring the card.\n");
		we_CloseCard(handle);//release card handle before exiting
		return(-1);
	}

	printf("Waiting for button to be pressed.  To exit, press Ctrl-C\n");
	fflush(stdout);//fflush would not normally be used, but it is in this case because of the continuous loop
	while(1)//loop to monitor the state of the pushbutton
	{		//Note that this loop has no delays, so it uses up
			//all of the CPU time that it can.  This is done to
			//keep the sample code simple. In a real program,
			//it is recommended that you implement some kind
			//of sleep or timer mechanism.

		result=we_InputValue(handle, 2, &value);
		if(result < 0)
		{
			printf("An error occurred while reading a value.\n");
			we_CloseCard(handle);//release card handle before exiting
			return(-1);
		}
		//in this sample, we will assume that when the button is pressed,
		//it results in Bit 0 being low (0)
		if( !(value & 1) )//test whether bit 0 (which has a binary weight of 1) is off
		{//the code below will execute if bit 0 was off (button pressed)
			count++;//increment counter
			printf("Count: %d\n", count);
			fflush(stdout);
			
			result=we_OutputValue(handle, 1, count);//output the count to the LEDs
			if(result < 0)
			{
				printf("An error occurred while writing a value.\n");
				we_CloseCard(handle);//release card handle before exiting
				return(-1);
			}
			//Now we will wait until bit 0 goes back to being on (wait til the pushbutton is released)
			//so, we will loop while bit 0 is off
			while( !(value & 1) )
			{
				we_InputValue(handle, 2, &value);
			}//end of inner while loop
		}//end of if (test for button pressed)
	}//end of while loop
	return(0);
}//end of main function

		
		
		
